/*
 * Example demonstrating use of dli.dll
 *
 * Copyright (C) 2012 Dennis Lee
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

typedef int (*cf_type)(int, int, int, int, int, int, int, int, unsigned char *, char *);
typedef int (*df_type)(unsigned char *, char *);
typedef int (*if_type)(int *, int *, int *, int *, int *, int *, char *);

static void
error_exit(char *str) {
	printf("Error: %s\n", str);
	exit(1);
}

int
main(int argc, char **argv) {
	HINSTANCE hlib;
	cf_type c_func;
	df_type d_func;
	if_type i_func;
	FILE *inf, *outf;
	int ii_w, ii_h, ii_gray, ii_qp, ii_qpc, ii_ac;
	unsigned char *imagebuf;

/* First step is loading the DLL, and getting addresses of the 3 functions. */

	hlib = LoadLibraryA("dli.dll");
	if (hlib == NULL)
		error_exit("Failed to load dli.dll");
	printf("\nLoaded dli.dll successfully.\n");
	c_func = (cf_type)GetProcAddress(hlib, "dli_dll_compress");
	d_func = (df_type)GetProcAddress(hlib, "dli_dll_decompress");
	i_func = (if_type)GetProcAddress(hlib, "dli_dll_info");
	if ((c_func == NULL) || (d_func == NULL) || (i_func == NULL))
		error_exit("Failed to get function addresses.");
	printf("Got function addresses successfully.\n");

/* For this example program, a fixed image (testimg) is used as input for compression.
   This image is 100x100 pixels and consists of raw RGB data.
   The image is loaded into imagebuf in the following block of code. */

	inf = fopen("testimg", "rb");
	if (inf == NULL)
		error_exit("Failed to open testimg.");
	imagebuf = malloc(30000);
	if (imagebuf == NULL)
		error_exit("Failed to allocate memory.");
	fread(imagebuf, 30000, 1, inf);
	fclose(inf);
	printf("Loaded testimg successfully.\n");

/* The image data in imagebuf is now compressed, and output as testimg.dli
   by calling c_func (dli_dll_compress in the DLL).  Refer to readme.txt
   for details on the compression parameters this function uses. */

	if (c_func(100, 100, 0, -1, -1, -1, -1, -1, imagebuf, "testimg.dli"))
		error_exit("Compression failed.");
	printf("Compression successful. (testimg.dli)\n");

/* The header of testimg.dli is read by calling i_func (dli_dll_info in the DLL).
   6 attributes of the image are returned, which are then printed on screen. */

	if (i_func(&ii_w, &ii_h, &ii_gray, &ii_qp, &ii_qpc, &ii_ac, "testimg.dli"))
		error_exit("Failed to read DLI header.");
	printf("Successfully read testimg.dli header:\n");
	printf("Image width: %d\n", ii_w);
	printf("Image height: %d\n", ii_h);
	printf("Grayscale flag: %d\n", ii_gray);
	printf("All chroma flag: %d\n", ii_ac);
	printf("-q value: %d\n", ii_qp);
	printf("-qc value: %d\n", ii_qpc);

/* testimg.dli is decompressed by calling d_func (dli_dll_decompress in the DLL).
   The decompressed image is written to imagebuf (a memory buffer), which
   happens to be allocated the correct size since it was used to hold the
   original image (testimg).  To decompress an arbitrary DLI image, use i_func to
   retrieve information on the image's size then allocate a properly sized buffer.
   The decompressed image data is then output to testimg.ppm in PPM format. */

	if (d_func(imagebuf, "testimg.dli"))
		error_exit("Corrupt DLI image.");
	outf = fopen("testimg.ppm", "wb");
	if (outf == NULL)
		error_exit("Failed to open testimg.ppm");
	fprintf(outf, "P6\n100 100\n255\n");
	fwrite(imagebuf, 30000, 1, outf);
	fclose(outf);
	printf("Decompression successful. (testimg.ppm)\n");

	free(imagebuf);
	FreeLibrary(hlib);
	return 0;
}
